// File:  SigCalc.h
// Glen Cowan
// RHUL Physics

#ifndef SIGCALC_H
#define SIGCALC_H

#include <iostream>
#include <vector>
#include <TRandom3.h>

using namespace std;

class SigCalc {

  public: 

    SigCalc (double n, double s, vector<double> mVec, 
             vector<double> tauVec, vector<double> S1Vec,
             vector<double> S2Vec, int option);
    SigCalc (double s, vector<double> tauVec, double mu, 
             vector<double> bVec, vector<double> aVec,
             vector<double> xiVec, TRandom3* ran, int option);
    double n() { return m_n; }
    double s() { return m_s; }
    double m(int i) { return m_m[i]; }
    double tau(int i) { return m_tau[i]; }
    double S1(int i) { return m_S1[i]; }
    double S2(int i) { return m_S2[i]; }
    int numBck(){ return m_numBck; }
    int option(){ return m_option; }
    double lnL(double mu, vector<double> bVec, vector<double> lambdaVec,
               vector<double> sigmaVec);
    double lnLLogNorm(double mu, vector<double> bVec, 
               vector<double> lambdaVec, vector<double> sigmaVec);
    double lnLGauss(double mu, vector<double> bVec, 
               vector<double> omegaVec, vector<double> sigmaVec);
    double lnLGaussBhat(double mu, vector<double> bVec);
    double lnLw1(double mu, vector<double> bVec);
    double qmu(double mu, double& muHat, vector<double>& bHat,
         vector<double>& bHatHat, vector<double>& lambdaHat,
               vector<double>& lambdaHatHat, vector<double>& sigmaHat,
               vector<double>& sigmaHatHat);
    double qmu(double mu);
    double q0(double& muHat, vector<double>& bHat,
        vector<double>& bHatHat, vector<double>& lambdaHat,
              vector<double>& lambdaHatHat, vector<double>& sigmaHat,
              vector<double>& sigmaHatHat);
    double q0();
    double psi(double, double);
    void setStartValues(vector<double>& parVec, vector<bool>& freePar);

  private:

    int            m_numBck;
    double         m_n;
    double         m_s;
    vector<double> m_m;
    vector<double> m_tau;
    vector<double> m_S1;
    vector<double> m_S2;

    int              m_option;

};

#endif
